using System;
using Microsoft.Web.Services;
using Microsoft.Web.Services.Diagnostics;
using Microsoft.Web.Services.Dime;
using Microsoft.Web.Services.Configuration;
using System.IO;
using System.Xml;
using System.Collections;
using System.Net;
using CDX_DOTNET_REQUESTOR;

using System.Threading;

namespace ExchangeNetwork
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class NodeRequestor
	{
		private NetworkNode requestor;

		public NodeRequestor()
		{
			init();			
		}

		public NodeRequestor(string Url)
		{			
			init();
			requestor.Url = Url;						
			
		}

		public NodeRequestor(string Url, ICertificatePolicy certificateValidator)
		{
			init();
			requestor.Url = Url;						
			//uses certificate validator of your choice
			ServicePointManager.CertificatePolicy = certificateValidator;			
			
		}
		

		private void init()
		{
			requestor = new NetworkNode();
			setOutputFilter();
			SoapContext context = requestor.RequestSoapContext;        
			context.Timestamp.Ttl = 0;		
			requestor.Timeout = Timeout.Infinite;			
			ServicePointManager.MaxServicePointIdleTime=Timeout.Infinite;
			ServicePointManager.CertificatePolicy = new DummyValidator();
		}

		private void setOutputFilter()
		{
			
			// Get the SoapOutputFilterCollection and add an instance
			// of a custom output filter that inherits from SoapOutputFilter.
			SoapOutputFilterCollection globalOutputFilters =
				WebServicesConfiguration.FilterConfiguration.OutputFilters;
			globalOutputFilters.Add(new TraceOutputFilter());
			globalOutputFilters.Add(new NodeRequestorOutputFilter());
			

			SoapInputFilterCollection globalInputFilters =
				WebServicesConfiguration.FilterConfiguration.InputFilters;
			globalInputFilters.Add(new TraceInputFilter());
			//globalInputFilters.Add(new NodeRequestorInputFilter());
		}

		private void SetMustUnderstand()
		{
			SoapContext context= requestor.RequestSoapContext;
			context.Path.EncodedMustUnderstand = "false";			
			//need to clean out attachments BEFORE EVERY CALL
			context.Attachments.Clear();
			//need to clean out CONTEXT BEFORE EVERY CALL
			context.Clear();
		}


		public void SetEndpointURL(String Url)
		{
			requestor.Url = Url;
		}

		public String NodePing(String Hello)
		{
			SetMustUnderstand();
			return requestor.NodePing(Hello);
		}

		public String Authenticate(String userId, String password)
		{
			SetMustUnderstand();
			return requestor.Authenticate(userId, password, "password");
		}

		public  void Download(String securityToken , String transactionId , String dataflow , ref NodeDocument[] documents)
		{
			SetMustUnderstand();

			requestor.Download(securityToken, transactionId, dataflow,  ref documents);

			SoapContext respContext= requestor.ResponseSoapContext;
			SoapEnvelope envelope = respContext.Envelope;

			XmlElement element = envelope.DocumentElement;
			Hashtable hashTable = new Hashtable();

			XmlNodeList list = element.GetElementsByTagName("content");

			//Now lets iterate over all elements in the list
        
			foreach (XmlNode node in list)
			{
				XmlAttributeCollection attributes = node.Attributes;
				String href = attributes["href"].Value;
				XmlNode parent = node.ParentNode;
				String documentName = parent["name"].InnerText;
				hashTable.Add(documentName, href);
			}

			
			for(int counter = 0 ; counter<documents.Length; counter++)
			{
				String href= (String) hashTable[documents[counter].name];
				DimeAttachment attachment= respContext.Attachments[href];

				BinaryReader br = new BinaryReader(attachment.Stream);

				Byte [] byteArray = br.ReadBytes((int)attachment.Stream.Length);
				documents[counter].content = byteArray;

			}


		}

		public string Submit(String securityToken, String transactionId, String dataflow, NodeDocument [] documents)
		{
			SetMustUnderstand();
			SoapContext reqContext= requestor.RequestSoapContext;
			Hashtable hash = new Hashtable();
        
			for(int i = 0 ; i< documents.Length ;i++)
			{

				DimeAttachment attachment = new DimeAttachment();
				attachment.TypeFormat = TypeFormatEnum.MediaType;
				attachment.Type = "application/octet-stream";
				//this needs to end up in the ref				 				
				attachment.Id = Guid.NewGuid().ToString();
				attachment.Stream = new MemoryStream(documents[i].content);				
				reqContext.Attachments.Add(attachment);
				hash.Add(documents[i].name, attachment.Id);
			}

			reqContext.Add("hrefs", hash);
			
			//requestor.
			
			//we need to add href and content

			return requestor.Submit(securityToken, transactionId, dataflow, documents);

		}
		public String[] GetServices(String securityToken , String serviceType )
		{
			SetMustUnderstand();
			return requestor.GetServices(securityToken, serviceType);
		}

		public String  GetStatus(String securityToken, String transactionId ) 
		{
			SetMustUnderstand();
			return requestor.GetStatus(securityToken, transactionId);
		}


		public String Query(String securityToken, String request, int rowId , int maxRows, String [] parameters)
		{
			SetMustUnderstand();
			return (String) requestor.Query(securityToken, request, Convert.ToString(rowId), Convert.ToString(maxRows), parameters);
		}


		public String Solicit(String securityToken , String returnURL , String request , String []  parameters)
		{
			SetMustUnderstand();
			return  requestor.Solicit(securityToken, returnURL, request, parameters);
		}

		public  String Notify(String securityToken , String nodeAddress , String dataflow, NodeDocument[] documents) 
		{
			SetMustUnderstand();

			SoapContext reqContext= requestor.RequestSoapContext;
			Hashtable hash = new Hashtable();
        
			for(int i = 0 ; i< documents.Length ;i++)
			{

				DimeAttachment attachment = new DimeAttachment();
				attachment.TypeFormat = TypeFormatEnum.MediaType;
				attachment.Type = "application/octet-stream";
				//this needs to end up in the ref				 				
				attachment.Id = Guid.NewGuid().ToString();
				attachment.Stream = new MemoryStream(documents[i].content);				
				reqContext.Attachments.Add(attachment);
				hash.Add(documents[i].name, attachment.Id);
			}

			reqContext.Add("hrefs", hash);

			return requestor.Notify(securityToken, nodeAddress, dataflow, documents);
		}

	}
}
